@@ -19,19 +19,23 @@
 ### Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 ### MA 02110-1301, USA
 
+## retrieve trellis object saved during plotting
 
-
-## retrieve last saved (while printing) trellis object
-
-trellis.last.object <- function(warn = TRUE, ...)
+trellis.last.object <- function(..., prefix = lattice.getStatus("current.prefix"))
 {
-    ans <- get("last.object", envir = .LatticeEnv)
-    if (is.null(ans)) {
-        warning("No trellis object currently saved")
-        return(invisible())
+    ## if (warn && (!lattice.getStatus("current.plot.saved", prefix = prefix)))
+    ##     warning("Requested object was not saved")
+    if (!lattice.getStatus("current.plot.saved", prefix = prefix))
+    {
+        warning("Requested 'trellis' object was not saved")
+        return(invisible(NULL))
     }
-    if (warn && !lattice.getStatus("current.plot.saved"))
-        warning("currently saved object is not the last one plotted")
+    ans <- lattice.getStatus("last.object", prefix = prefix)
+    ## FIXME: remove, as should not be needed any more
+    ## if (is.null(ans)) { 
+    ##     warning("Could not retrieve saved trellis object")
+    ##     return(invisible())
+    ## }
     update(ans, ...)
 }
 
@@ -235,11 +239,9 @@ update.trellis <-
     }
     if (!missing(panel))
     {
-        panel <- 
-            if (is.function(panel)) panel 
-            else if (is.character(panel)) get(panel)
-            else eval(panel)
-        if (as.character(object$call[[1]]) == "splom")
+        panel <- getFunctionOrName(panel)
+        ## if (as.character(object$call[[1]]) == "splom")
+        if ("panel" %in% names(object$panel.args.common))
             object$panel.args.common$panel <- panel
         else object$panel <- panel
     }
@@ -406,17 +408,14 @@ update.trellis <-
     if (!missing(prepanel))
     {
         recalculateLimits <- TRUE
-        prepanel <-
-            if (is.function(prepanel)) prepanel 
-            else if (is.character(prepanel)) get(prepanel)
-            else eval(prepanel)
+        ## prepanel <- getFunctionOrName(prepanel)
     }
     else prepanel <- object$prepanel
 
     if (recalculateLimits)
     {
         prepanel.def <- object$prepanel.default
-        laa <- limits.and.aspect(prepanel.default.function = prepanel.def,
+        laa <- limits.and.aspect(prepanel.def,
                                  prepanel = prepanel,
                                  have.xlim = have.xlim,
                                  xlim = xlim,
