@@ -74,12 +74,11 @@ panel.xyplot <-
              lwd = if (is.null(groups)) plot.line$lwd else superpose.line$lwd,
              horizontal = FALSE,
              ...,
+             grid = FALSE, abline = NULL,
              jitter.x = FALSE, jitter.y = FALSE,
              factor = 0.5, amount = NULL)
 {
     if (all(is.na(x) | is.na(y))) return()
-    x <- as.numeric(x)
-    y <- as.numeric(y)
     plot.symbol <- trellis.par.get("plot.symbol")
     plot.line <- trellis.par.get("plot.line")
     superpose.symbol <- trellis.par.get("superpose.symbol")
@@ -89,6 +88,22 @@ panel.xyplot <-
         if (missing(col.line)) col.line <- col
         if (missing(col.symbol)) col.symbol <- col
     }
+    if (missing(grid) && ("g" %in% type)) grid <- TRUE ## FIXME: what if list?
+    if (!identical(grid, FALSE))
+    {
+        if (!is.list(grid))
+            grid <- switch(as.character(grid),
+                           "TRUE" = list(h = -1, v = -1, x = x, y = y),
+                           "h" = list(h = -1, v = 0, y = y),
+                           "v" = list(h = 0, v = -1, x = x),
+                           list(h = 0, v = 0))
+        do.call(panel.grid, grid)
+    }
+    if (!is.null(abline))
+    {
+        if (is.numeric(abline)) abline <- as.list(abline)
+        do.call(panel.abline, abline)
+    }
     if (!is.null(groups))
         panel.superpose(x, y,
                         type = type,
@@ -109,11 +124,13 @@ panel.xyplot <-
                         jitter.y = jitter.y,
                         factor = factor,
                         amount = amount,
+                        grid = FALSE, ## grid=TRUE/type="g" already handled
                         ...)
     else
     {
+        x <- as.numeric(x)
+        y <- as.numeric(y)
         if ("o" %in% type || "b" %in% type) type <- c(type, "p", "l")
-        if ("g" %in% type) panel.grid(h = -1, v = -1)
         if ("p" %in% type)
             panel.points(x = if (jitter.x) jitter(x, factor = factor, amount = amount) else x,
                          y = if (jitter.y) jitter(y, factor = factor, amount = amount) else y,
@@ -227,6 +244,7 @@ xyplot.formula <-
              ...,
              lattice.options = NULL,
              default.scales = list(),
+             default.prepanel = lattice.getOption("prepanel.default.xyplot"),
              subscripts = !is.null(groups),
              subset = TRUE)
 {
@@ -254,12 +272,6 @@ xyplot.formula <-
     
     if ("subscripts" %in% names(formals(panel))) subscripts <- TRUE
     if (subscripts) subscr <- form$subscr
-
-    prepanel <-
-        if (is.function(prepanel)) prepanel
-        else if (is.character(prepanel)) get(prepanel)
-        else eval(prepanel)
-
     cond <- form$condition
     y <- form$left
     x <- form$right
@@ -309,16 +321,16 @@ xyplot.formula <-
     ## Step 3: Decide if limits were specified in call:
 
     have.xlim <- !missing(xlim)
-    if (!is.null(foo$x.scales$limit)) # override xlim
+    if (!is.null(foo$x.scales$limits)) # override xlim
     {
         have.xlim <- TRUE
-        xlim <- foo$x.scales$limit
+        xlim <- foo$x.scales$limits
     }
     have.ylim <- !missing(ylim)
-    if (!is.null(foo$y.scales$limit))
+    if (!is.null(foo$y.scales$limits))
     {
         have.ylim <- TRUE
-        ylim <- foo$y.scales$limit
+        ylim <- foo$y.scales$limits
     }
 
     ## Step 4: Decide if log scales are being used:
@@ -390,7 +402,7 @@ xyplot.formula <-
     ## FIXME: make this adjustment everywhere else
 
     more.comp <-
-        c(limits.and.aspect(prepanel.default.xyplot,
+        c(limits.and.aspect(default.prepanel,
                             prepanel = prepanel, 
                             have.xlim = have.xlim, xlim = xlim, 
                             have.ylim = have.ylim, ylim = ylim, 
